@@ -33,13 +33,21 @@
 #'           \item "output": Same as "estimate" but also writes summary of choices into temporary file (later read by \code{apollo_modelOutput}).
 #'           \item "raw": Same as "prediction".
 #'         }
+#' @importFrom stats setNames
 apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avail, continuousChoice, budget, functionality, minConsumption=NA, rows="all"){
   
+  # ############################## #
+  #### functionality="validate" ####
+  # ############################## #
+  
   if(functionality=="validate"){
     
+    # Store useful values
     nObs  <- length(continuousChoice[[1]])
     nAlts <- length(V)
+    avail_set <- FALSE
     
+    # check rows statement
     if(length(rows)!=nObs & !(length(rows)==1 && rows=="all")) stop("The argument \"rows\" needs to either be \"all\" or a boolean vector of length equal to the number of the rows in the data!")
     if(length(rows)==1 && rows=="all") rows <- rep(TRUE, nObs) else {
       if(length(budget)==length(continuousChoice[[1]])) continuousChoice[[1]][!rows] = budget[!rows] else {
@@ -48,6 +56,12 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
       for(j in 2:nAlts) continuousChoice[[j]][!rows] = 0
     }
     
+    # Create availability if needed
+    if(!is.list(avail)){
+      avail_set <- TRUE
+      avail <- setNames(as.list(rep(1,nAlts)), alternatives)
+    }
+    
     discrete_choice=list()
     k=1
     while(k<= nAlts){
@@ -55,6 +69,7 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
       k=k+1
     }
     
+    # set minConsumption to NA if they are all zero
     if(!anyNA(minConsumption)) if(Reduce('+',lapply(minConsumption, sum))==0) minConsumption=NA
     
     apollo_control <- tryCatch( get("apollo_inputs", parent.frame(), inherits=TRUE )$apollo_control,
@@ -62,15 +77,20 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
     
     if(apollo_control$noValidation==FALSE){
       
+      # Check that sigma is not random (actually, it could be, but it leads to weird results)
       if(!is.vector(sigma)) stop("Sigma should not be random")
       if(length(sigma)!=1 && length(sigma)!=nObs) stop("Sigma should be either a scalar or a vector with as many elements as observations")
       
+      # Check there are at least two alternatives
       if(nAlts<2) stop("MDCEV requires at least two products")
       
+      # Check that choice vector is not empty
       if(nObs==0) stop("No choices to model")  
       
+      # Check that first product is outside good
       if(alternatives[1]!="outside") stop("First product must be called \"outside\"!")
       
+      # Check labels
       if(!all(alternatives %in% names(V))) stop("Labels in \"alternatives\" do not match those in \"V\"!")
       if(!all(alternatives %in% names(alpha))) stop("Labels in \"alternatives\" do not match those in \"alpha\"!")
       if(!all(alternatives %in% names(gamma))) stop("Labels in \"alternatives\" do not match those in \"gamma\"!")
@@ -78,6 +98,7 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
       if(!all(alternatives %in% names(cost))) stop("Labels in \"alternatives\" do not match those in \"cost\"!")
       if(!all(alternatives %in% names(avail))) stop("Labels in \"alternatives\" do not match those in \"avail\"!")
       
+      # check that nothing unavailable is chosen
       chosenunavail=0
       j=1
       while(j<= length(V)){
@@ -85,27 +106,36 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
         j=j+1}
       if(chosenunavail==1) stop("Some product(s) chosen despite being listed as unavailable!")
       
+      # check that outside good is always chosen
       if(sum(continuousChoice[[1]]==0)>0) stop("First product should always be chosen as it is an outside good!")
       
+      # checks for consumption for individual products
       budget_check=0*budget
       
       k=1
       while(k<= length(V)){
+        # check that all costs are positive
         if(sum(cost[[k]]==0)>0) stop("Need strictly positive costs for all products!")
         
+        # check that no negative consumptions for any products
         if(sum(continuousChoice[[k]]<0)>0) stop("Negative consumption for some products for some observations!")
         budget_check=budget_check+continuousChoice[[k]]*cost[[k]]
         k=k+1
       }
       
+      # check that full budget is consumed in each row, nothing more, nothing less
       if(sum(abs(budget_check-budget)>10^-10)) stop("Expenditure for some observations is either less or more than budget!")
       
+      # turn scalar availabilities into vectors
       for(i in 1:length(avail)) if(length(avail[[i]])==1) avail[[i]] <- rep(avail[[i]], nObs)
       
+      # check that all availabilities are either 0 or 1
       for(i in 1:length(avail)) if( !all(unique(avail[[i]]) %in% 0:1) ) stop("Some availability values are not 0 or 1.")
       
+      # check that if minimum consumption exists, it has the same names as alternatives, and that no consumptions are less than minConsumption if alternative is available
       if(!anyNA(minConsumption)){
         if(!all(alternatives %in% names(minConsumption))) stop("Labels in \"alternatives\" do not match those in \"minConsumption\"!")
+        # first ensure order is correct
         if(any(alternatives != names(continuousChoice))) continuousChoice <- continuousChoice[alternatives]
         if(any(alternatives != names(minConsumption))) minConsumption <- minConsumption[alternatives]
         consumption_below_min_flag=0
@@ -117,7 +147,7 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
         if(consumption_below_min_flag) stop("\nSome consumptions are below the lower limits listed in \"minConsumption\"!")
       }
       
-      cat("\nAll checks passed for MDEV model component\n")
+      #cat("\nAll checks passed for MDCEV model component\n")
     }
     
     if(apollo_control$noDiagnostics==FALSE){  
@@ -137,17 +167,28 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
       rownames(choicematrix) = c("Times available","Observations in which chosen","Average consumption when available","Average consumption when chosen")
       colnames(choicematrix) = names(V)
       
-      cat('Overview of choices for MDEV model component:\n')
-      print(round(choicematrix,2))
-      cat("\n")
+      #cat('Overview of choices for MDEV model component:\n')
+      #print(round(choicematrix,2))
+      #cat("\n")
+      #if(any(choicematrix[2,]==0)) cat("Warning: some alternatives are never chosen in your data!\n")
+      #if(any(choicematrix[2,]==1)) cat("Warning: some alternatives are always chosen when available!\n")
       
-      if(any(choicematrix[2,]==0)) cat("Warning: some alternatives are never chosen in your data!\n")
-      if(any(choicematrix[2,]==1)) cat("Warning: some alternatives are always chosen when available!\n")
+      content <- list(round(choicematrix,2))
+      if(any(choicematrix[4,]==0)) content[[length(content) + 1]] <- "Warning: some alternatives are never chosen in your data!"
+      if(any(choicematrix[4,]==1)) content[[length(content) + 1]] <- "Warning: some alternatives are always chosen when available!"
+      if(avail_set==TRUE) content[[length(content)+1]] <- paste0("Warning: Availability not provided (or some elements are NA).",
+                                                                 "\n", paste0(rep(" ",9),collapse=""),"Full availability assumed.")
+      apolloLog <- tryCatch(get("apollo_inputs", parent.frame(), inherits=TRUE )$apolloLog, error=function(e) return(NA))
+      apollo_addLog("Overview of choices for MDCEV model component:", content, apolloLog)
       
     }
     return(TRUE)
   }
   
+  # ############################## #
+  #### functionality="zero_LL" ####
+  # ############################## #
+  
   if(functionality=="zero_LL"){
     nObs  <- length(continuousChoice[[1]])
     if(length(rows)==1 && rows=="all") rows <- rep(TRUE, nObs) else {
@@ -161,8 +202,13 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
     return(P)
   }
   
+  # ############################################### #
+  #### functionality="estimate/conditionals/raw" ####
+  # ################################################# #
+  
   if(functionality%in%c("estimate","conditionals","raw")){
     
+    # Store useful values
     nObs  <- length(continuousChoice[[1]])
     nAlts <- length(V)
     if(length(rows)==1 && rows=="all") rows <- rep(TRUE, nObs) else {
@@ -172,6 +218,12 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
       for(j in 2:nAlts) continuousChoice[[j]][!rows] = 0
     }
     
+    # Create availability if needed
+    if(!is.list(avail)){
+      avail_set <- TRUE
+      avail <- setNames(as.list(rep(1,nAlts)), alternatives)
+    }
+    
     discrete_choice=list()
     k=1
     while(k<= nAlts){
@@ -179,8 +231,10 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
       k=k+1
     }
     
+    # set minConsumption to NA if they are all zero
     if(!anyNA(minConsumption)) if(Reduce('+',lapply(minConsumption, sum))==0) minConsumption=NA
     
+    # Reorder V and avail to match alternatives order, if necessary
     if(any(alternatives != names(V))) V <- V[alternatives]
     if(any(alternatives != names(avail))) avail <- avail[alternatives]
     if(any(alternatives != names(alpha))) alpha <- alpha[alternatives]
@@ -190,6 +244,7 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
     if(any(alternatives != names(cost))) cost <- cost[alternatives]
     if(!anyNA(minConsumption)) if(any(alternatives != names(minConsumption))) minConsumption <- minConsumption[alternatives]
     
+    # Compute V
     V[[1]]=(alpha[[1]]-1)*log(continuousChoice[[1]])
     j=2
     while(j<=length(V)){
@@ -202,10 +257,13 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
       j=j+1
     }
     
+    # Get M (number of different chosen products)
     totalChosen=Reduce("+",discrete_choice)
     
+    # First term
     term1=(1-totalChosen)*log(sigma)
     
+    # Second term
     logfi=list()
     j=2
     while(j<=length(V)){
@@ -225,6 +283,7 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
       j=j+1
     }
     
+    # Third term
     term3=continuousChoice[[1]]/(1-alpha[[1]])
     j=2
     while(j<=length(V)){
@@ -232,6 +291,7 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
       j=j+1}
     term3 = log(term3)
     
+    # Fourth term
     term4_1 = V[[1]]/sigma
     term4_2 = exp(V[[1]]/sigma)
     j=2
@@ -243,8 +303,10 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
     term4_2 = totalChosen * log(term4_2) 
     term4 =  term4_1 - term4_2 
     
+    # Fifth term: log of factorial
     term5 = lfactorial(totalChosen-1)
     
+    # probability is simply the exp of the sum of the logs of the individual terms
     P = exp(term1 + term2 + term3 + term4 + term5)
     if(is.vector(P)) P[!rows]   <- 1
     if(is.matrix(P)) P[!rows,]  <- 1
@@ -254,12 +316,18 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
     
   }
   
+  # ############################## #
+  #### functionality="output" ####
+  # ############################## #
+  
   if(functionality=="output"){
     
     P <- apollo_mdcevOutside(V, alternatives, alpha, gamma, sigma, cost, avail, continuousChoice, budget, functionality="estimate", minConsumption, rows)
     
+    # Store useful values
     nObs  <- length(continuousChoice[[1]])
     nAlts <- length(V)
+    avail_set <- FALSE
     if(length(rows)==1 && rows=="all") rows <- rep(TRUE, nObs) else {
       if(length(budget)==length(continuousChoice[[1]])) continuousChoice[[1]][!rows] = budget[!rows] else {
         continuousChoice[[1]][!rows] = budget
@@ -267,6 +335,12 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
       for(j in 2:nAlts) continuousChoice[[j]][!rows] = 0
     }
     
+    # Create availability if needed
+    if(!is.list(avail)){
+      avail_set <- TRUE
+      avail <- setNames(as.list(rep(1,nAlts)), alternatives)
+    }
+    
     discrete_choice=list()
     k=1
     while(k<= nAlts){
@@ -274,8 +348,10 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
       k=k+1
     }
     
+    # set minConsumption to NA if they are all zero
     if(!anyNA(minConsumption)) if(Reduce('+',lapply(minConsumption, sum))==0) minConsumption=NA
     
+    # Reorder V and avail to match alternatives order, if necessary
     if(any(alternatives != names(V))) V <- V[alternatives]
     if(any(alternatives != names(avail))) avail <- avail[alternatives]
     if(any(alternatives != names(alpha))) alpha <- alpha[alternatives]
@@ -285,6 +361,7 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
     if(any(alternatives != names(cost))) cost <- cost[alternatives]
     if(!anyNA(minConsumption)) if(any(alternatives != names(minConsumption))) minConsumption <- minConsumption[alternatives]
     
+    # turn scalar availabilities into vectors
     for(i in 1:length(avail)) if(length(avail[[i]])==1) avail[[i]] <- rep(avail[[i]], nObs)
     
     availprint = colSums(matrix(unlist(avail), ncol = length(avail))[rows,])
@@ -303,35 +380,49 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
     rownames(choicematrix) = c("Times available","Observations in which chosen","Average consumption when available","Average consumption when chosen")
     colnames(choicematrix) = names(V)
     
-    apollo_control <- tryCatch( get("apollo_inputs", parent.frame(), inherits=TRUE )$apollo_control,
-                                error=function(e){
-                                  cat("apollo_mdcev could not retrieve apollo_control. No diagnostics in output.\n")
-                                  return(NA)
-                                } )
-    if(!(length(apollo_control)==1 && is.na(apollo_control))){
-      fileName <- paste(apollo_control$modelName, "_tempOutput.txt", sep="")
-      fileName <- file.path(tempdir(),fileName)
-      fileConn <- tryCatch( file(fileName, open="at"),
-                            error=function(e){
-                              cat('apollo_mdcev could not write diagnostics to temporary file. No diagnostics in output.\n')
-                              return(NA)
-                            })
-      if(!anyNA(fileConn)){
-        sink(fileConn)
-        on.exit({if(sink.number()>0) sink(); close(fileConn)})
-        if(apollo_control$noDiagnostics==FALSE){
-          cat('\nOverview of choices for MDEV model component:\n')
-          print(round(choicematrix,2))
-          cat('\n')}
-        if(sum(choicematrix[2,]==0)>0) cat("Warning: some alternatives are never chosen in your data!\n")
-      }
-    }
+    ## write diagnostics to a file named "modelName_tempOutput.txt" in a temporary directory.
+    #apollo_control <- tryCatch( get("apollo_inputs", parent.frame(), inherits=TRUE )$apollo_control,
+    #                            error=function(e){
+    #                              cat("apollo_mdcev could not retrieve apollo_control. No diagnostics in output.\n")
+    #                              return(NA)
+    #                            } )
+    #if(!(length(apollo_control)==1 && is.na(apollo_control))){
+    #  fileName <- paste(apollo_control$modelName, "_tempOutput.txt", sep="")
+    #  fileName <- file.path(tempdir(),fileName)
+    #  fileConn <- tryCatch( file(fileName, open="at"),
+    #                        error=function(e){
+    #                          cat('apollo_mdcev could not write diagnostics to temporary file. No diagnostics in output.\n')
+    #                          return(NA)
+    #                        })
+    #  if(!anyNA(fileConn)){
+    #    sink(fileConn)
+    #    on.exit({if(sink.number()>0) sink(); close(fileConn)})
+    #    if(apollo_control$noDiagnostics==FALSE){
+    #      cat('\nOverview of choices for MDEV model component:\n')
+    #      print(round(choicematrix,2))
+    #      cat('\n')}
+    #    if(sum(choicematrix[2,]==0)>0) cat("Warning: some alternatives are never chosen in your data!\n")
+    #  }
+    #}
+    
+    content <- list(round(choicematrix,2))
+    if(any(choicematrix[4,]==0)) content[[length(content) + 1]] <- "Warning: some alternatives are never chosen in your data!"
+    if(any(choicematrix[4,]==1)) content[[length(content) + 1]] <- "Warning: some alternatives are always chosen when available!"
+    if(avail_set==TRUE) content[[length(content)+1]] <- paste0("Warning: Availability not provided (or some elements are NA).",
+                                                               "\n", paste0(rep(" ",9),collapse=""),"Full availability assumed.")
+    apolloLog <- tryCatch(get("apollo_inputs", parent.frame(), inherits=TRUE )$apolloLog, error=function(e) return(NA))
+    apollo_addLog("Overview of choices for MDCEV model component:", content, apolloLog)
     
     return(P)
   }
   
+  # ################################ #
+  #### functionality="prediction" ####
+  # ################################ #
+  
   if(functionality=="prediction"){
     
+    # Store useful values
     nObs  <- length(continuousChoice[[1]])
     nAlts <- length(V)
     if(length(rows)==1 && rows=="all") rows <- rep(TRUE, nObs) else {
@@ -341,8 +432,10 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
       for(j in 2:nAlts) continuousChoice[[j]][!rows] = 0
     }
     
+    # set minConsumption to NA if they are all zero
     if(!anyNA(minConsumption)) if(Reduce('+',lapply(minConsumption, sum))==0) minConsumption=NA
     
+    # Reorder V and avail to match alternatives order, if necessary
     if(any(alternatives != names(V))) V <- V[alternatives]
     if(any(alternatives != names(avail))) avail <- avail[alternatives]
     if(any(alternatives != names(alpha))) alpha <- alpha[alternatives]
@@ -350,6 +443,7 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
     if(any(alternatives != names(continuousChoice))) continuousChoice <- continuousChoice[alternatives]
     if(any(alternatives != names(cost))) cost <- cost[alternatives]
     
+    # first include a check to make sure that all alphas are equal across alts
     equality_check=1
     j=2
     while(j<= length(alpha)){
@@ -358,6 +452,7 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
     }
     if(equality_check!=1) stop("\nMDCEV prediction only implemented for profile where alpha is generic across all products!")
     
+    # include a check to make sure there are no minimum consumption limits
     if(!anyNA(minConsumption)) stop("\nMDCEV prediction only implemented for case without non-zero minimum consumptions!")
     
     forecasting=function(V,alternatives,alpha,gamma,sigma,cost,avail,budget,continuousChoice)
@@ -368,6 +463,9 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
       nObs=length(continuousChoice[[1]])[1]
       consumption_total=array(0,dim=c(nObs,length(alternatives),ND))
       
+      # create uniform draws for value draws
+      # this needs to change still if we also have random alpha, gamma or V
+      
       Ndraws=apollo_mlhs(ND*nObs,length(alternatives),1)
       draws_base=cbind(x=rep(seq(1,ND),nObs),Ndraws)
       
@@ -397,6 +495,9 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
         for(i in 1:length(gamma)) if(length(gamma[[i]])==1) gamma[[i]] <- rep(gamma[[i]], nObs)
         for(i in 1:length(cost)) if(length(cost[[i]])==1) cost[[i]] <- rep(cost[[i]], nObs)
         
+        # need a check here to see if we're using a given column for example of the cube
+        
+        # turn into matrices
         V_trans_use=Reduce("cbind",V_trans)
         alpha_use=Reduce("cbind",alpha)
         gamma_use=Reduce("cbind",gamma)
@@ -405,6 +506,7 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
         
         consumption_total[,,k]=forecasting_subroutine(V_trans_use,alpha_use,gamma_use,avail_use,cost_use,budget,continuousChoice)
         
+        # increment draws  
         if(k%%round(ND/10,0)==0) if(k==round(ND/2,0)) cat("50%") else if(k==ND) cat("100%") else cat(".")
         k=k+1
       }
@@ -427,12 +529,14 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
         use=(orderofV<M)
         while(stopping<1)
         {
+          #step2
           lambda_1=((budget[i]+sum(cost[i,(2:ncol(cost))]*gamma[i,(2:ncol(gamma))]*use)))
           lambda_21=(V_trans[i,1])^(1/(1-alpha[i,1]))
           lambda_22=sum(cost[i,(2:ncol(cost))]*gamma[i,(2:ncol(gamma))]*use*(V_trans[i,2:NALT])^(1/(1-alpha[i,1])))
           lambda=(lambda_1/(lambda_21+lambda_22))^(alpha[i,1]-1)
           if((sum(lambda<V_trans[i,2:NALT]))<(M))
           {              
+            #step3
             consumption_outside_1=(V_trans[i,1])^(1/(1-alpha[i,1]))*(budget[i]+sum(use*cost[i,(2:ncol(cost))]*gamma[i,(2:ncol(gamma))]))
             consumption_outside_2=lambda_21+lambda_22
             consumption[i,1]=consumption_outside_1/consumption_outside_2
@@ -443,6 +547,7 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
           }
           else
           {
+            #step4
             M=M+1  
             use=(orderofV<M)
             if(M==(sum(avail[i,2:NALT])+1))
@@ -503,6 +608,7 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
       
       cat("You have mixing")
       
+      # check for intra-respondent, and if it exists, give a message, and average out third dimension. applies to all inputs (V,alpha,gamma,sigma)
       
       Ninter = apollo_inputs$apollo_draws$interNDraws
       
@@ -524,7 +630,7 @@ apollo_mdcevOutside <- function(V, alternatives, alpha, gamma, sigma, cost, avai
         consumption_overall[,,,r] = forecasting(Vdraw,alternatives,alphadraw,gammadraw,sigma,cost,avail,budget,continuousChoice)
         r=r+1
       }
-      consumption <- apply(consumption_overall, MARGIN=c(1,2), mean) 
+      consumption <- apply(consumption_overall, MARGIN=c(1,2), mean) # Average intra-draws
       consumption[!rows, ] <- NA
       return( consumption )
     }
