@@ -80,56 +80,73 @@ apollo_nl <- function(nl_settings, functionality){
   if(is.null(nl_settings[["nlNests"]])) stop("The nl_settings list needs to include an object called \"nlNests\"!")
   if(is.null(nl_settings[["nlStructure"]])) stop("The nl_settings list needs to include an object called \"nlStructure\"!")
   if(is.null(nl_settings[["rows"]])) nl_settings[["rows"]]="all"
-
-  alternatives=nl_settings[["alternatives"]]
-  avail=nl_settings[["avail"]]
-  choiceVar=nl_settings[["choiceVar"]]
-  V=nl_settings[["V"]]
-  rows=nl_settings[["rows"]]
-  nlNests=nl_settings[["nlNests"]]
-  nlStructure=nl_settings[["nlStructure"]]
-
+  
+  alternatives = nl_settings[["alternatives"]]
+  avail        = nl_settings[["avail"]]
+  choiceVar    = nl_settings[["choiceVar"]]
+  V            = nl_settings[["V"]]
+  rows         = nl_settings[["rows"]]
+  nlNests      = nl_settings[["nlNests"]]
+  nlStructure  = nl_settings[["nlStructure"]]
+  
+  # ############################## #
+  #### functionality="validate" ####
+  # ############################## #
+  
   if(functionality=="validate"){
+    # Store useful values
     apollo_control <- tryCatch(get("apollo_control", parent.frame(), inherits=FALSE),
-                            error = function(e) list(noValidation=FALSE, noDiagnostics=FALSE))
+                               error = function(e) list(noValidation=FALSE, noDiagnostics=FALSE))
     nObs  <- length(choiceVar)
     nAlts <- length(V)
     avail_set <- FALSE
+    root_set  <- FALSE
     altnames=names(alternatives)
     altcodes=alternatives
     nestnames <- names(nlStructure)
     if(!("root" %in% names(nlNests))){
+      root_set <- TRUE
       nlNests["root"] <- 1
-      warning("Root lambda parameter set to 1.")
+      #warning("Root lambda parameter set to 1.")
     }
     if(length(rows)==1 && rows=="all") rows=rep(TRUE,length(choiceVar))
     choiceVar[!rows]=alternatives[1]
-
+    
+    # check rows statement
     if(length(rows)!=length(choiceVar)) stop("The argument \"rows\" needs to either be \"all\" or a vector of length equal to the number of the rows in the data!")
-
-
+    
+    # Check that alternatives are named in altcodes and V
+    if(is.null(altnames) || is.null(altcodes) || is.null(names(V))) stop("Alternatives must be named, both in 'altnernatives' and 'V'.")
+    
+    # Create availability if necessary
     if(!is.list(avail)){
       avail_set <- TRUE
       avail <- vector(mode="list", length=nAlts)
       avail <- lapply(avail, function(a) 1)
       names(avail) <- altnames
-      warning("Full availability of alternatives assumed for NL component.")
+      #warning("Full availability of alternatives assumed for NL component.")
     }
+    # Reorder V to match altnames order, if necessary
     if(any(altnames != names(V))) V <- V[altnames]
     if(any(altnames != names(avail))) avail <- avail[altnames]
-
+    
     if(apollo_control$noValidation==FALSE){
-
+      
+      # Check there are at least three alternatives
       if(nAlts<3) stop("NL requires at least three alternatives")
-
+      
+      # Check that choice vector is not empty
       if(nObs==0) stop("No choices to model")
-
+      
+      # Check that labels in choice match those in the utilities and availabilities
       choiceLabs <- unique(choiceVar)
       if(!all(altnames %in% names(V))) stop("Alternative labels in \"altnames\" do not match those in \"V\".")
       if(!all(altnames %in% names(avail))) stop("Alternative labels in \"altnames\" do not match those in \"avail\".")
-
+      
+      # Check that there are no values in the choice column for undefined alternatives
       if(!all(choiceLabs %in% altcodes)) stop("Value in choice column that is not included in altcodes.")
-
+      
+      # check that nothing unavailable is chosen
       chosenunavail=0
       j=1
       while(j <= length(altnames)){
@@ -137,18 +154,20 @@ apollo_nl <- function(nl_settings, functionality){
         j=j+1
       }
       if(chosenunavail==1) stop("Some alternative(s) chosen despite being listed as unavailable\n")
-
+      
+      # check that all availabilities are either 0 or 1
       for(i in 1:length(avail)) if( !all(unique(avail[[i]]) %in% 0:1) ) stop("Some availability values are not 0 or 1.")
-
-
-
+      
+      
+      # checks that are specific to nlStructure component
+      
       allElements <- c("root", unlist(nlStructure))
       if(nlNests["root"]!=1) stop("The root lambda parameter should be equal to 1.")
       if( !all(altnames %in% allElements) ) stop("All alternatives must be included in the tree structure.")
       if( !all(nestnames %in% allElements) ) stop("All nests must be included in the tree structure.")
       if( (length(nestnames)+length(altnames))!=length(allElements) ) stop("Tree structure is inconsistent. Each element must appear only once.")
       if( !all(names(nlNests) %in% names(nlStructure)) | !all(names(nlStructure) %in% names(nlNests)) ) stop("All nests in argument 'nlNests' should be in 'nlStructure', and vice versa (including 'root').")
-
+      
       if(is.null(nlStructure[["root"]])) stop("Tree structure is missing an element called root!")
       combined_elements="root"
       j=1
@@ -156,27 +175,28 @@ apollo_nl <- function(nl_settings, functionality){
         combined_elements=c(combined_elements,nlStructure[[j]])
         j=j+1
       }
-
+      
       j=1
       while(j<= length(altnames)){
         if(sum(nestnames==altnames[j])) stop("A nest cannot have the same name as an alternative!")
         if(sum(combined_elements==altnames[j])!=1) stop("An alternative needs to appear exactly once in a tree!")
         j=j+1
       }
-
+      
       j=1
       while(j<= length(nlStructure)){
         if(sum(nestnames==names(nlStructure)[j])!=1) stop("A defined nest needs to appear exactly once in a tree!")
         j=j+1
       }
-
+      
       j=1
       while(j<= length(nestnames)){
         if(sum(altnames==nestnames[j])) stop("A nest cannot have the same name as an alternative!")
         if(sum(combined_elements==nestnames[j])!=1) stop("A defined nest needs to appear exactly once in a tree!")
         j=j+1
       }
-
+      
+      # reorder nlStructure if needed
       nlStructure_ordered=list()
       element_list="root"
       j=1
@@ -195,35 +215,34 @@ apollo_nl <- function(nl_settings, functionality){
         }
         j=length(element_list)
       }
-
+      
       nlStructure=nlStructure_ordered
-
-      cat("\nAll checks passed for NL model component\n")
+      
+      # confirm all checks are passed, print output and return TRUE
+      #cat("\nAll checks passed for NL model component\n")
     }
     if(apollo_control$noDiagnostics==FALSE){
       
-      if(avail_set==TRUE) warning("Availability not provided to 'apollo_nl' (or some elements are NA). Full availability assumed.")
+      #if(avail_set==TRUE) warning("Availability not provided to 'apollo_nl' (or some elements are NA). Full availability assumed.")
+      # turn scalar availabilities into vectors
       for(i in 1:length(avail)) if(length(avail[[i]])==1) avail[[i]] <- rep(avail[[i]], nObs)
-
-      availprint = colSums(rows*matrix(unlist(avail), ncol = length(avail))) 
+      
+      # Construct summary table of availabilities and market share
+      availprint = colSums(rows*matrix(unlist(avail), ncol = length(avail))) # number of times each alt is available
       choicematrix = matrix(0,nrow=4,ncol=length(altnames))
       choicematrix[1,] = availprint
       j=1
       while(j<= length(altnames)){
-        choicematrix[2,j]=sum(choiceVar==altcodes[j] & rows) 
+        choicematrix[2,j]=sum(choiceVar==altcodes[j] & rows) # number of times each alt is chosen
         j=j+1
       }
-      choicematrix[3,] = choicematrix[2,]/sum(rows)*100 
-      choicematrix[4,] = choicematrix[2,]/choicematrix[1,]*100 
+      choicematrix[3,] = choicematrix[2,]/sum(rows)*100 # market share
+      choicematrix[4,] = choicematrix[2,]/choicematrix[1,]*100 # market share controlled by availability
       choicematrix[4,!is.finite(choicematrix[4,])] <- 0
       rownames(choicematrix) = c("Times available","Times chosen","Percentage chosen overall","Percentage chosen when available")
       colnames(choicematrix) = altnames
-      cat('Overview of choices for MNL model component:\n')
-      print(round(choicematrix,2))
-      cat("\n")
-      if(any(choicematrix[4,]==0)) cat("Warning: some alternatives are never chosen in your data!\n")
-      if(any(choicematrix[4,]==1)) cat("Warning: some alternatives are always chosen when available!\n")
-
+      
+      # Order tree structure
       nlStructure_ordered=list()
       element_list="root"
       j=1
@@ -243,7 +262,8 @@ apollo_nl <- function(nl_settings, functionality){
         j=length(element_list)
       }
       nlStructure=nlStructure_ordered
-
+      
+      # Calculate ancestors
       ancestors=list()
       j=1
       while(j<= length(altnames)){
@@ -260,41 +280,71 @@ apollo_nl <- function(nl_settings, functionality){
         }
         j=j+1
       }
-
+      
+      # Load function to print tree structure and print tree structure
       print_tree=function(nlStructure, ancestors){
-
+        
         print_tree_level = function(nlStructure, component, preceding_nest_layer, space){
           j=1
           if(preceding_nest_layer!=0) space=c(space,"  |")
           while(j<=length(nlStructure[[component]])){
-            space <- gsub("[']", " ", space) 
+            space <- gsub("[']", " ", space)
             if(j==length(nlStructure[[component]])) space[length(space)] <- gsub("[|]", "'", space[length(space)])
             if(nlStructure[[component]][j] %in% altnames){
               depth <- length(space)
               cat("\n",space,rep("-",3*(maxDepth-depth)),"-Alternative: ",nlStructure[[component]][j], sep="")
             } else {
-              cat("\n", space, "-Nest: ", nlStructure[[component]][j], sep="")
+              #cat("\n", space, "-Nest: ", nlStructure[[component]][j], sep="")
+              cat("\n",space,"-Nest: ",nlStructure[[component]][j]," (",round(nlNests[[nlStructure[[component]][j]]],4), ")", sep="")
               print_tree_level(nlStructure, nlStructure[[component]][j], preceding_nest_layer+1, space)
             }
             j=j+1
           }
         }
-
+        
         maxDepth <- max(sapply(ancestors, length))-1
-        cat("\nNest: ",names(nlStructure)[[1]], sep="")
-
+        #cat("\nNest: ",names(nlStructure)[[1]], sep="")
+        cat("Nest: ",names(nlStructure)[[1]]," (",round(nlNests[[names(nlStructure)[[1]]]],4),")", sep="")
+        
         print_tree_level(nlStructure, "root", preceding_nest_layer=0, space="|")
       }
-      cat("Nested logit structure:")
-      print_tree(nlStructure, ancestors)
-      cat("\n")
-
+      
+      
+      ### Printing diagnostics
+      #cat('Overview of choices for NL model component:\n')
+      #print(round(choicematrix,2))
+      #cat("\n")
+      #if(any(choicematrix[4,]==0)) cat("Warning: some alternatives are never chosen in your data!\n")
+      #if(any(choicematrix[4,]==1)) cat("Warning: some alternatives are always chosen when available!\n")
+      #cat("Nested logit structure:")
+      #print_tree(nlStructure, ancestors)
+      #cat("\n")
+      content <- list(round(choicematrix,2))
+      if(any(choicematrix[4,]==0)) content[[length(content)+1]] <- "Warning: some alternatives are never chosen in your data!"
+      if(any(choicematrix[4,]==1)) content[[length(content)+1]] <- "Warning: some alternatives are always chosen when available!"
+      if(avail_set==TRUE) content[[length(content)+1]] <- paste0("Warning: Availability not provided (or some elements are NA).",
+                                                                 "\n", paste0(rep(" ",9),collapse=""),"Full availability assumed.")
+      if(root_set==TRUE) content[[length(content)+1]] <- "Notice: Root lambda parameter set to 1."
+      content[[length(content)+1]] <- "Nested structure:"
+      content[[length(content)+1]] <- capture.output(print_tree(nlStructure, ancestors))
+      #content <- list(round(choicematrix,2),
+      #                ifelse(any(choicematrix[4,]==0), "Warning: some alternatives are never chosen in your data!", ""),
+      #                ifelse(any(choicematrix[4,]==1), "Warning: some alternatives are always chosen when available!", ""),
+      #                "Nested structure:",
+      #                capture.output(print_tree(nlStructure, ancestors)))
+      apolloLog <- tryCatch(get("apollo_inputs", parent.frame(), inherits=TRUE )$apolloLog, error=function(e) return(NA))
+      apollo_addLog("Overview of choices for NL model component:", content, apolloLog)
     }
-
+    
     return(TRUE)
   }
-
+  
+  # ############################## #
+  #### functionality="zero_LL" ####
+  # ############################## #
+  
   if(functionality=="zero_LL"){
+    # Store useful values
     nObs  <- length(choiceVar)
     nAlts <- length(V)
     avail_set <- FALSE
@@ -304,24 +354,30 @@ apollo_nl <- function(nl_settings, functionality){
     if(!("root" %in% names(nlNests))) nlNests["root"] <- 1
     if(length(rows)==1 && rows=="all") rows=rep(TRUE,length(choiceVar))
     choiceVar[!rows]=alternatives[1]
-
+    
+    # Create availability if necessary
     if(!is.list(avail)){
       avail_set <- TRUE
       avail <- vector(mode="list", length=nAlts)
       avail <- lapply(avail, function(a) 1)
       names(avail) <- altnames
     }
-
+    
     if(!anyNA(avail)) if(any(altnames != names(avail))) avail <- avail[altnames]
-    for(i in 1:length(avail)) if(length(avail[[i]])==1) avail[[i]] <- rep(avail[[i]], nObs) 
-    nAvAlt <- rowSums(matrix(unlist(avail), ncol = length(avail))) 
-    P = 1/nAvAlt 
+    for(i in 1:length(avail)) if(length(avail[[i]])==1) avail[[i]] <- rep(avail[[i]], nObs) # turn scalar availabilities into vectors
+    nAvAlt <- rowSums(matrix(unlist(avail), ncol = length(avail))) # number of available alts in each observation
+    P = 1/nAvAlt # likelihood at zero
     P[!rows] <- 1
     return(P)
   }
-
+  
+  # ############################################################ #
+  #### functionality="estimate/prediction/conditionals/raw" ####
+  # ############################################################ #
+  
   if(functionality %in% c("estimate","prediction","conditionals","raw")){
-
+    
+    # Store useful values
     if(functionality=="raw" && length(choiceVar)==1 && is.na(choiceVar)) choiceVar = alternatives[1]
     nObs  <- length(choiceVar)
     nAlts <- length(V)
@@ -332,24 +388,27 @@ apollo_nl <- function(nl_settings, functionality){
     if(!("root" %in% names(nlNests))) nlNests["root"] <- 1
     if(length(rows)==1 && rows=="all") rows=rep(TRUE,length(choiceVar))
     choiceVar[!rows]=alternatives[1]
-
+    
+    # Create availability if necessary
     if(!is.list(avail)){
       avail_set <- TRUE
       avail <- vector(mode="list", length=nAlts)
       avail <- lapply(avail, function(a) 1)
       names(avail) <- altnames
     }
-
+    
+    ### Reorder V and avail to match altnames order, if necessary
     if(any(altnames != names(V))) V <- V[altnames]
     if(!anyNA(avail)) if(any(altnames != names(avail))) avail <- avail[altnames]
-
+    
     combined_elements="root"
     j=1
     while(j<= length(nlStructure)){
       combined_elements=c(combined_elements,nlStructure[[j]])
       j=j+1
     }
-
+    
+    # Order tree structure
     nlStructure_ordered=list()
     element_list="root"
     j=1
@@ -369,19 +428,22 @@ apollo_nl <- function(nl_settings, functionality){
       j=length(element_list)
     }
     nlStructure=nlStructure_ordered
-
+    
+    # extract chosen utility
     chosenV <- Reduce('+',
                       lapply(as.list(1:nAlts),
                              FUN=function(i) (choiceVar==altcodes[i])*V[[altnames[i]]])
     )
-
+    
+    # substract chosen utility from all others for numerical stability
     V = lapply(X=V, FUN=function(v) v-chosenV)
-
+    
+    # loop over nests to create new utility elements and new availability terms
     k=length(nlStructure)
     while(k>0){
       nestK <- names(nlStructure)[k]
       V[[nestK]] = 0
-      avail[[nestK]] = 1*( Reduce('+', avail[ nlStructure[[k]] ]) > 0 ) 
+      avail[[nestK]] = 1*( Reduce('+', avail[ nlStructure[[k]] ]) > 0 ) # calculate availability of nest
       j = 1
       while(j<= length(nlStructure[[k]])){
         nodeJ <- nlStructure[[k]][j]
@@ -391,7 +453,8 @@ apollo_nl <- function(nl_settings, functionality){
       V[[nestK]] = nlNests[[nestK]]*log(V[[nestK]])
       k = k-1
     }
-
+    
+    # work out ancestors
     ancestors=list()
     j=1
     while(j<= length(altnames)){
@@ -408,14 +471,15 @@ apollo_nl <- function(nl_settings, functionality){
       }
       j=j+1
     }
-
+    
+    # calculate log(probabilities)
     logPalts=list()
     j=1
     while(j <= length(altnames)){
       logPalts[[j]]=0
       k=1
       ancestorsJ <- ancestors[[altnames[[j]]]]
-      while(k< length(ancestorsJ)){ 
+      while(k< length(ancestorsJ)){ # loop to level just below root
         current_V = V[[ ancestorsJ[k] ]]
         next_V    = V[[ ancestorsJ[k+1] ]]
         logPalts[[j]] = logPalts[[j]] + (current_V-next_V)/nlNests[[ ancestorsJ[k+1] ]]
@@ -423,19 +487,22 @@ apollo_nl <- function(nl_settings, functionality){
       }
       j=j+1
     }
-
+    
     Palts = lapply(X=logPalts, FUN=exp)
-
+    
     tempnames=names(V)
     names(Palts)=tempnames[1:length(altnames)]
-
+    
+    # consider availabilities (it assumes Palts and avail are in the same order)
     Palts <- mapply('*', Palts, avail[1:length(altnames)], SIMPLIFY = FALSE)
-
+    
     if(functionality=="prediction"|(functionality=="raw")){
       Palts <- lapply(Palts, function(x) {x[is.na(x)] <- 0
-                         return(x)}) 
-      if(functionality=="prediction") Palts[["chosen"]] = Reduce('+', tmp <- mapply(function(x,y) (choiceVar==y)*x, Palts, as.list(altcodes), SIMPLIFY=FALSE) ) 
+      return(x)}) # replace all NaN by 0
+      # add an additional column with chosen
+      if(functionality=="prediction") Palts[["chosen"]] = Reduce('+', tmp <- mapply(function(x,y) (choiceVar==y)*x, Palts, as.list(altcodes), SIMPLIFY=FALSE) ) # keep only probability for chosen alternative
       Palts <- lapply(Palts, function(p) {
+        # change likelihood of excluded columns for 1
         if(is.vector(p)) p[!rows]  <- ifelse(functionality=="prediction",NA,1)
         if(is.matrix(p)) p[!rows,] <- ifelse(functionality=="prediction",NA,1)
         if(is.array(p) & length(dim(p))==3) p[!rows,,] <- ifelse(functionality=="prediction",NA,1)
@@ -443,45 +510,57 @@ apollo_nl <- function(nl_settings, functionality){
       })
     } else {
       Palts <- lapply(Palts, function(x) {x[is.na(x)] <- 0
-      return(x)}) 
-      Palts = Reduce('+', tmp <- mapply(function(x,y) (choiceVar==y)*x, Palts, as.list(altcodes), SIMPLIFY=FALSE) ) 
+      return(x)}) # replace all NaN by 0
+      Palts = Reduce('+', tmp <- mapply(function(x,y) (choiceVar==y)*x, Palts, as.list(altcodes), SIMPLIFY=FALSE) ) # keep only probability for chosen alternative
       if(is.vector(Palts)) Palts[!rows]  <- 1
       if(is.matrix(Palts)) Palts[!rows,] <- 1
       if(is.array(Palts) && length(dim(Palts))==3) Palts[!rows,,] <- 1
     }
-
+    
     return(Palts)
   }
-
+  
+  # ############################## #
+  #### functionality="output" ####
+  # ############################## #
+  
   if(functionality=="output"){
-
+    
+    # Store useful values
     nObs  <- length(choiceVar)
     nAlts <- length(V)
     avail_set <- FALSE
+    root_set  <- FALSE
     altnames=names(alternatives)
     altcodes=alternatives
     nestnames <- names(nlStructure)
-    if(!("root" %in% names(nlNests))) nlNests["root"] <- 1
+    if(!("root" %in% names(nlNests))){
+      root_set <- TRUE
+      nlNests["root"] <- 1
+    }
     if(length(rows)==1 && rows=="all") rows=rep(TRUE,length(choiceVar))
     choiceVar[!rows]=alternatives[1]
-
+    
+    # Create availability if necessary
     if(!is.list(avail)){
       avail_set <- TRUE
       avail <- vector(mode="list", length=nAlts)
       avail <- lapply(avail, function(a) 1)
       names(avail) <- altnames
     }
-
+    
+    ### Reorder V and avail to match altnames order, if necessary
     if(any(altnames != names(V))) V <- V[altnames]
     if(!anyNA(avail)) if(any(altnames != names(avail))) avail <- avail[altnames]
-
+    
     combined_elements="root"
     j=1
     while(j<= length(nlStructure)){
       combined_elements=c(combined_elements,nlStructure[[j]])
       j=j+1
     }
-
+    
+    # Order tree structure
     nlStructure_ordered=list()
     element_list="root"
     j=1
@@ -501,19 +580,22 @@ apollo_nl <- function(nl_settings, functionality){
       j=length(element_list)
     }
     nlStructure=nlStructure_ordered
-
+    
+    # extract chosen utility
     chosenV <- Reduce('+',
                       lapply(as.list(1:nAlts),
                              FUN=function(i) (choiceVar==altcodes[i])*V[[altnames[i]]])
     )
-
+    
+    # substract chosen utility from all others for numerical stability
     V = lapply(X=V, FUN=function(v) v-chosenV)
-
+    
+    # loop over nests to create new utility elements and new availability terms
     k=length(nlStructure)
     while(k>0){
       nestK <- names(nlStructure)[k]
       V[[nestK]] = 0
-      avail[[nestK]] = 1*( Reduce('+', avail[ nlStructure[[k]] ]) > 0 ) 
+      avail[[nestK]] = 1*( Reduce('+', avail[ nlStructure[[k]] ]) > 0 ) # calculate availability of nest
       j = 1
       while(j<= length(nlStructure[[k]])){
         nodeJ <- nlStructure[[k]][j]
@@ -523,7 +605,8 @@ apollo_nl <- function(nl_settings, functionality){
       V[[nestK]] = nlNests[[nestK]]*log(V[[nestK]])
       k = k-1
     }
-
+    
+    # work out ancestors
     ancestors=list()
     j=1
     while(j<= length(altnames)){
@@ -540,14 +623,15 @@ apollo_nl <- function(nl_settings, functionality){
       }
       j=j+1
     }
-
+    
+    # calculate log(probabilities)
     logPalts=list()
     j=1
     while(j <= length(altnames)){
       logPalts[[j]]=0
       k=1
       ancestorsJ <- ancestors[[altnames[[j]]]]
-      while(k< length(ancestorsJ)){ 
+      while(k< length(ancestorsJ)){ # loop to level just below root
         current_V = V[[ ancestorsJ[k] ]]
         next_V    = V[[ ancestorsJ[k+1] ]]
         logPalts[[j]] = logPalts[[j]] + (current_V-next_V)/nlNests[[ ancestorsJ[k+1] ]]
@@ -555,44 +639,47 @@ apollo_nl <- function(nl_settings, functionality){
       }
       j=j+1
     }
-
+    
     Palts = lapply(X=logPalts, FUN=exp)
-
+    
     tempnames=names(V)
     names(Palts)=tempnames[1:length(altnames)]
-
+    
+    # consider availabilities (it assumes Palts and avail are in the same order)
     Palts <- mapply('*', Palts, avail[1:length(altnames)], SIMPLIFY = FALSE)
-
+    
     Palts <- lapply(Palts, function(x) {x[is.na(x)] <- 0
-    return(x)}) 
-    P = Reduce('+', tmp <- mapply(function(x,y) (choiceVar==y)*x, Palts, as.list(altcodes), SIMPLIFY=FALSE) ) 
+    return(x)}) # replace all NaN by 0
+    P = Reduce('+', tmp <- mapply(function(x,y) (choiceVar==y)*x, Palts, as.list(altcodes), SIMPLIFY=FALSE) ) # keep only probability for chosen alternative
     if(is.vector(P)) P[!rows]  <- 1
     if(is.matrix(P)) P[!rows,] <- 1
     if(is.array(P) && length(dim(P))==3) P[!rows,,,drop=FALSE] <- 1
-
+    
+    # Construct summary table of availabilities and market share
     avail <- avail[altnames]
-    for(i in 1:length(avail)) if(length(avail[[i]])==1) avail[[i]] <- rep(avail[[i]], nObs) 
-    availprint = colSums(rows*matrix(unlist(avail), ncol = length(avail))) 
+    for(i in 1:length(avail)) if(length(avail[[i]])==1) avail[[i]] <- rep(avail[[i]], nObs) # turn scalar availabilities into vectors
+    availprint = colSums(rows*matrix(unlist(avail), ncol = length(avail))) # n times each alt is available
     choicematrix = matrix(0,nrow=4,ncol=length(altnames))
     choicematrix[1,] = availprint
     j=1
     while(j<= length(altnames)){
-      choicematrix[2,j]=sum(choiceVar==altcodes[j] & rows) 
+      choicematrix[2,j]=sum(choiceVar==altcodes[j] & rows) # times chosen
       j=j+1
     }
-    choicematrix[3,] = choicematrix[2,]/sum(rows)*100 
-    choicematrix[4,] = choicematrix[2,]/choicematrix[1,]*100 
+    choicematrix[3,] = choicematrix[2,]/sum(rows)*100 # market share
+    choicematrix[4,] = choicematrix[2,]/choicematrix[1,]*100 # market share controlled by availability
     choicematrix[4,!is.finite(choicematrix[4,])] <- 0
     rownames(choicematrix) = c("Times available","Times chosen","Percentage chosen overall","Percentage chosen when available")
     colnames(choicematrix) = altnames
-
+    
+    # Load function to write tree structure
     print_tree=function(nlStructure, ancestors){
-
+      
       print_tree_level = function(nlStructure, component, preceding_nest_layer, space){
         j=1
         if(preceding_nest_layer!=0) space=c(space,"  |")
         while(j<=length(nlStructure[[component]])){
-          space <- gsub("[']", " ", space) 
+          space <- gsub("[']", " ", space)
           if(j==length(nlStructure[[component]])) space[length(space)] <- gsub("[|]", "'", space[length(space)])
           if(nlStructure[[component]][j] %in% altnames){
             depth <- length(space)
@@ -604,42 +691,59 @@ apollo_nl <- function(nl_settings, functionality){
           j=j+1
         }
       }
-
+      
       maxDepth <- max(sapply(ancestors, length))-1
-      cat("\nNest: ",names(nlStructure)[[1]]," (",round(nlNests[[names(nlStructure)[[1]]]],4),")", sep="")
-
+      cat("Nest: ",names(nlStructure)[[1]]," (",round(nlNests[[names(nlStructure)[[1]]]],4),")", sep="")
+      
       print_tree_level(nlStructure, "root", preceding_nest_layer=0, space="|")
     }
-
-    apollo_control <- tryCatch( get("apollo_inputs", parent.frame(), inherits=FALSE )$apollo_control,
-                                error=function(e){
-                                  cat("apollo_nl could not retrieve apollo_control. No diagnostics in output.\n")
-                                  return(NA)
-                                } )
-    if(!(length(apollo_control)==1 && is.na(apollo_control))){
-      fileName <- paste(apollo_control$modelName, "_tempOutput.txt", sep="")
-      fileName <- file.path(tempdir(),fileName)
-      fileConn <- tryCatch( file(fileName, open="at"),
-                            error=function(e){
-                              cat('apollo_nl could not write diagnostics to temporary file. No diagnostics in output.\n')
-                              return(NA)
-                            })
-      if(!anyNA(fileConn)){
-        sink(fileConn)
-        on.exit({if(sink.number()>0) sink(); close(fileConn)})
-        if(apollo_control$noDiagnostics==FALSE){
-          cat('Overview of choices for NL model component:\n')
-          print(round(choicematrix,0))
-          cat("\n")
-        }
-        cat("Structure of nested logit component:")
-        print_tree(nlStructure, ancestors)
-        cat('\n')
-        if(sum(choicematrix[4,]==0)>0) cat("Warning: some alternatives are never chosen in your data!\n")
-      }
-    }
-
+    
+    ## write diagnostics to a file named "modelName_tempOutput.txt" in a temporary directory.
+    #apollo_control <- tryCatch( get("apollo_inputs", parent.frame(), inherits=FALSE )$apollo_control,
+    #                            error=function(e){
+    #                              cat("apollo_nl could not retrieve apollo_control. No diagnostics in output.\n")
+    #                              return(NA)
+    #                            } )
+    #if(!(length(apollo_control)==1 && is.na(apollo_control))){
+    #  fileName <- paste(apollo_control$modelName, "_tempOutput.txt", sep="")
+    #  fileName <- file.path(tempdir(),fileName)
+    #  fileConn <- tryCatch( file(fileName, open="at"),
+    #                        error=function(e){
+    #                          cat('apollo_nl could not write diagnostics to temporary file. No diagnostics in output.\n')
+    #                          return(NA)
+    #                        })
+    #  if(!anyNA(fileConn)){
+    #    sink(fileConn)
+    #    on.exit({if(sink.number()>0) sink(); close(fileConn)})
+    #    if(apollo_control$noDiagnostics==FALSE){
+    #      cat('Overview of choices for NL model component:\n')
+    #      print(round(choicematrix,0))
+    #      cat("\n")
+    #    }
+    #    cat("Structure of nested logit component:")
+    #    print_tree(nlStructure, ancestors)
+    #    cat('\n')
+    #    if(sum(choicematrix[4,]==0)>0) cat("Warning: some alternatives are never chosen in your data!\n")
+    #  }
+    #}
+    
+    #content <- list(round(choicematrix,2),
+    #                ifelse(any(choicematrix[4,]==0), "Warning: some alternatives are never chosen in your data!", ""),
+    #                ifelse(any(choicematrix[4,]==1), "Warning: some alternatives are always chosen when available!", ""),
+    #                "Final nesting structure:",
+    #                capture.output(print_tree(nlStructure, ancestors)))
+    content <- list(round(choicematrix,2))
+    if(any(choicematrix[4,]==0)) content[[length(content)+1]] <- "Warning: some alternatives are never chosen in your data!"
+    if(any(choicematrix[4,]==1)) content[[length(content)+1]] <- "Warning: some alternatives are always chosen when available!"
+    if(avail_set==TRUE) content[[length(content)+1]] <- paste0("Warning: Availability not provided (or some elements are NA).",
+                                                               "\n", paste0(rep(" ",9),collapse=""),"Full availability assumed.")
+    if(root_set==TRUE) content[[length(content)+1]] <- "Notice: Root lambda parameter set to 1."
+    content[[length(content)+1]] <- "Nested structure:"
+    content[[length(content)+1]] <- capture.output(print_tree(nlStructure, ancestors))
+    apolloLog <- tryCatch(get("apollo_inputs", parent.frame(), inherits=TRUE )$apolloLog, error=function(e) return(NA))
+    apollo_addLog("Overview of choices for NL model component:", content, apolloLog)
+    
   }
-
+  
   return(P)
 }
